## METADATA ===============================================================
## Hagen et al. Earth history events shaped the evolution of uneven biodiversity across tropical moist forests. PNAS
## Description: Config template for gen3sis experiment
## 
## R version: 4.0.0 for Windows
## Date: 2020-07-14 16:20:32
## License: GPL3
## Author: Oskar Hagen (oskar@hagen.bio) and Alexander Skeels (alexander.skeels@gmail.com)
##=======================================================================##


########################
### General settings ###
########################
# set the random seed for the simulation
random_seed = params$seed

# set the starting time step or leave NA to use the earliest/highest timestep
start_time = params$start

# set the end time step or leave as NA to use the lates/lowest timestep (0)
end_time = NA

# maximum total number of species in the simulation before it is aborted
max_number_of_species = 12500

# maximum number of species within one cell before the simulation is aborted
max_number_of_coexisting_species = 12500

# a list of traits to include with each species
# a "dispersal" trait is implictly added in any case # can add precipitation for global
trait_names = c("temp", "temp_width", "prec", "dispersal")

# ranges to scale the input environemts with:
# not listed variable:         no scaling takes place
# listed, set to NA:           the environmental variable will be scaled from [min, max] to [0, 1]
# lsited with a given range r: the environmental variable will be scaled from [r1, r2] to [0, 1]

# use the absolute ma and min as scaling factors
environmental_ranges = list("temp" = c(-37.62053, 27.98271), "prec"= NA)

# a place to inspect the internal state of the simulation and collect additional information if desired
end_of_timestep_observer = function(data, vars, config){
  par(mfrow=c(1,1))
# plot species richness
  plot_richness(data$all_species, data$landscape)
# save everything
  save_richness()
  save_phylogeny()
  save_species()
  save_divergence()
  save_abundance()
  save_landscape()
  save_traits()
  
  
  # create and save presence absence matrice
  if(!file.exists(file.path(config$directories$output, "pa_matrices"))){dir.create(file.path(config$directories$output, "pa_matrices"))}
  pa_matrix <- data.frame(matrix(0,nrow=length(data$landscape$coordinates[,1]), ncol=(length(data$all_species)+2)))
  pa_matrix[,1:2]<-data$landscape$coordinates
  rownames(pa_matrix) <- rownames(data$landscape$coordinates)
  names(pa_matrix)[1:2] <- c("x", "y")
  names(pa_matrix)[3:length(pa_matrix)] <-unlist(lapply(data$all_species, FUN=function(x){x$id}))
  for(i in 3:(length(pa_matrix[1,]))){
    pa_matrix[names(data$all_species[[i-2]]$abundance),i] <- 1
  }
  write.table(pa_matrix, file.path(config$directories$output,"pa_matrices",  paste0("species_pa_",vars$ti, ".txt")), row.names = F, col.names = T)

}


######################
### Initialization ###
######################

# the intial abundace of a newly colonized cell, both during setup and later when colonizing a cell during the dispersal
initial_abundance = params$initialAbundance

create_ancestor_species <- function(landscape, config) {

  p_world <- raster::rasterFromXYZ(cbind(landscape$coordinates[which(landscape$coordinates[,2] > -23.5 & landscape$coordinates[,2] < 23.5),], 
                                         landscape$environment[which(landscape$coordinates[,2] > -23.5 & landscape$coordinates[,2] < 23.5), 1, drop = F]))
  p_world <- extend(p_world, landscape$extent)
  mesic_equatorial_cells <- as.character(Which(p_world, cells=T))
  
  all_species <- list()
  new_species <- create_species(NA, as.character(mesic_equatorial_cells), config)
  new_species$traits[ , "dispersal"] <- 1
  new_species$traits[ , "temp"] <- landscape$environment[mesic_equatorial_cells, "temp"]
  new_species$traits[ , "prec"] <- 0
  new_species$traits[ , "temp_width"] <- params$TW
  all_species <- append(all_species, list(new_species))

  return(all_species)
}


#################
### Dispersal ###
#################
# returns n dispersal values

get_dispersal_values <- function(n, species, landscape, config) {
  dispersal_range = c(0, params$grid_cell_distance)
  weighted_dispersal <- sum(species[["traits"]][, "dispersal"] * species[["abundance"]])
  disp_factor <- weighted_dispersal/sum(species[["abundance"]])
  scale <- ((dispersal_range[2] - dispersal_range[1]) * disp_factor ) + dispersal_range[1]

  values <- rweibull(n, shape = params$WeibullShape, scale = scale)
  return(values)
}

##################
### Speciation ###
##################
# threshold for genetic distance after which a speciation event takes place
divergence_threshold = params$S

# factor by which the divergence is increased between geographicaly isolated population
# can also be a matrix between the different population clusters
#adds 1 to each disconnected cluster
get_divergence_factor <- function(species, cluster_indices, landscape, config) {
  return(0.1)
}


#######################
### Trait Evolution ###
#######################
# mutate the traits of a species and return the new traits matrix
apply_evolution <- function(species, cluster_indices, landscape, config) {
  
  traits <- species[["traits"]]
  traits[, "temp"] <- traits[, "temp"] + rnorm(1, mean = 0, sd = params$Tev)
  # set bounds between 0 and 1 so the species can;t evolve a niche beyond that present in the data (all temp is scaled between 0 and 1)
  if(any(traits[, "temp"] > 1)){traits[which(traits[,"temp"]>1), "temp"] <- 1}
  if(any(traits[, "temp"] < 0)){traits[which(traits[,"temp"]<0), "temp"] <- 0}
  return(traits)
  
}




###############
### Ecology ###
###############
# called for every cell with all occuring species, this functin calculates the who survives in the current cells
# returns a vector of abundances
# set the abundance to 0 for every species supposed to die
# performs for each grid cell/population seperately
# landscape is the value of the grid cell

apply_ecology <- function(abundance, traits, landscape, config) {
  temp_diff <- abs( traits[, "temp"] - landscape[, "temp"])
  abundance <- temp_diff < traits[, "temp_width"] 
  return(abundance)
}


